# services/review_fetcher.py

from typing import Dict
from src.reviewsense_ecom.service.retrieval import get_vector_store
from src.reviewsense_ecom.service.feature_extractor import FeatureExtractor
from typing import (Any)


class RatingsFetcher:
    """Class for fetching reviews from the vector store"""

    def __init__(self):
        self.vector_store = get_vector_store()
        self.feature_extractor = FeatureExtractor()

    def fetch_ratings(self, product_id: str) -> Dict[str, Any]:
        rating_distribution = {}
        filter_criteria = {"title": product_id}
        documents = self.vector_store.metadata_search(filter=filter_criteria, n=100)
        ratings_count = len(documents)

        total_rating = 0
        rated_documents = 0
        total_reviews = []

        for doc in documents:
            total_reviews.append(doc.page_content.lstrip("Review: "))
            if 'rating' in doc.metadata:
                rating = str(doc.metadata['rating'])
                rating_distribution[rating] = rating_distribution.get(rating, 0) + 1
                total_rating += float(rating)
                rated_documents += 1

        # Calculate average rating, handling case where there are no ratings
        average_rating = round(total_rating / rated_documents, 2) if rated_documents > 0 else 0

        return {
            "ratings_count": ratings_count,
            "average_rating": average_rating,
            "rating_distribution": rating_distribution,
            "total_reviews": total_reviews
        }
